package org.apache.bcel.generic;

import java.io.IOException;
import java.util.Iterator;
import java.io.DataOutputStream;
import org.apache.bcel.classfile.ElementValue;
import java.util.ArrayList;
import org.apache.bcel.classfile.ArrayElementValue;
import java.util.List;

public class ArrayElementValueGen extends ElementValueGen
{
    private final List<ElementValueGen> evalues;
    
    public ArrayElementValueGen(final ArrayElementValue value, final ConstantPoolGen cpool, final boolean copyPoolEntries) {
        super(91, cpool);
        this.evalues = new ArrayList<ElementValueGen>();
        final ElementValue[] elementValuesArray;
        final ElementValue[] in = elementValuesArray = value.getElementValuesArray();
        for (final ElementValue element : elementValuesArray) {
            this.evalues.add(ElementValueGen.copy(element, cpool, copyPoolEntries));
        }
    }
    
    public ArrayElementValueGen(final ConstantPoolGen cp) {
        super(91, cp);
        this.evalues = new ArrayList<ElementValueGen>();
    }
    
    public ArrayElementValueGen(final int type, final ElementValue[] datums, final ConstantPoolGen cpool) {
        super(type, cpool);
        if (type != 91) {
            throw new IllegalArgumentException("Only element values of type array can be built with this ctor - type specified: " + type);
        }
        this.evalues = new ArrayList<ElementValueGen>();
        for (final ElementValue datum : datums) {
            this.evalues.add(ElementValueGen.copy(datum, cpool, true));
        }
    }
    
    public void addElement(final ElementValueGen gen) {
        this.evalues.add(gen);
    }
    
    @Override
    public void dump(final DataOutputStream dos) throws IOException {
        dos.writeByte(super.getElementValueType());
        dos.writeShort(this.evalues.size());
        for (final ElementValueGen element : this.evalues) {
            element.dump(dos);
        }
    }
    
    @Override
    public ElementValue getElementValue() {
        final ElementValue[] immutableData = new ElementValue[this.evalues.size()];
        int i = 0;
        for (final ElementValueGen element : this.evalues) {
            immutableData[i++] = element.getElementValue();
        }
        return new ArrayElementValue(super.getElementValueType(), immutableData, this.getConstantPool().getConstantPool());
    }
    
    public List<ElementValueGen> getElementValues() {
        return this.evalues;
    }
    
    public int getElementValuesSize() {
        return this.evalues.size();
    }
    
    @Override
    public String stringifyValue() {
        final StringBuilder sb = new StringBuilder();
        sb.append("[");
        String comma = "";
        for (final ElementValueGen element : this.evalues) {
            sb.append(comma);
            comma = ",";
            sb.append(element.stringifyValue());
        }
        sb.append("]");
        return sb.toString();
    }
}
